@@ -1,5 +1,8 @@
+# -*- coding: utf-8 -*-
+
 import slumber
 
+
 class Client(object):
 
     class_instance = None
@@ -32,8 +35,6 @@ class Client(object):
 
         return cls.class_instance
 
-
-
     def collection(self, name):
         """
         """
@@ -63,12 +64,20 @@ class SimpleQuery(object):
 
 class Query(object):
 
+    SORTING_ASC = 'ASC'
+    SORTING_DESC = 'DESC'
+
     def __init__(self):
         """
         """
 
         self.collections = []
+        self.filters = []
 
+        self.start = -1
+        self.count = -1
+
+        self.sortings = []
 
     def append_collection(self, collection_name):
         """
@@ -76,6 +85,49 @@ class Query(object):
 
         self.collections.append(collection_name)
 
+        return self
+
+    def filter(self, **kwargs):
+
+        for key, value in kwargs:
+
+            splitted_filter = key.split('__')
+
+            if len(splitted_filter) is 1:
+
+                self.filters.append(
+                    QueryFilterStatement(
+                        collection=self.collections[-1],
+                        attribute=key,
+                        operator=QueryFilterStatement.EQUAL_OPERATOR,
+                        value=value,
+                    )
+                )
+
+            else:
+
+                self.filters.append(
+                    QueryFilterStatement(
+                        collection=splitted_filter[0],
+                        attribute=splitted_filter[1],
+                        operator=QueryFilterStatement.EQUAL_OPERATOR,
+                        value=value,
+                    )
+                )
+
+        return self
+
+    def limit(count, start=-1):
+        self.start = start
+        self.count = count
+
+    def order_by(field, order=Query.SORTING_ASC, collection=None):
+
+        self.sortings.append({
+            'field': field,
+            'order': order,
+            'collection': collection,
+        })
 
     def execute(self):
         """
@@ -84,7 +136,52 @@ class Query(object):
         query_data = ''
 
         for collection in self.collections:
-            query_data += 'FOR %s in %s' % ( collection + '_123', collection )
+            query_data += 'FOR %s in %s' % (
+                self._get_collection_ident(collection),
+                collection
+            )
+
+        for filter_statement in self.filters:
+            query_data += 'FILTER %s.%s %s %s' % (
+                filter_statement.collection,
+                filter_statement.attribute,
+                filter_statement.operator,
+                filter_statement.value,
+            )
+
+        is_first = True
+
+        for sorting_entry in self.sorting:
+
+            if is_first:
+                query_data += 'SORT '
+
+                is_first = False
+
+            if sorting_entry['field'] is not None:
+
+                if not is_first:
+                    query_data += ', '
+
+                if sorting_entry['collection'] is not None:
+                    query_data += '%s.%s %s' % (
+                        self._get_collection_ident(sorting_entry['collection']),
+                        sorting_entry['field'],
+                        sorting_entry['order'],
+                    )
+                else:
+                    query_data += '%s.%s %s' % (
+                        self._get_collection_ident(collection[0]),
+                        sorting_entry['field'],
+                        sorting_entry['order'],
+                    )
+
+        if self.count is not -1:
+
+            if self.start is not -1:
+                query_data += 'LIMIT %s, %s' % (self.start, self.count)
+            else:
+                query_data += 'LIMIT %s' % self.count
 
         query_data += 'RETURN %s' % collection + '_123'
 
@@ -94,10 +191,13 @@ class Query(object):
 
         api = Client.instance().api
 
-        result = api.query.post(data=post_data)
+        result = api.cursor.post(data=post_data)
 
         return  result
 
+    def _get_collection_ident(collection_name):
+        return collection_name + '_123'
+
 
 class Traveser(object):
 
@@ -172,7 +272,6 @@ class Database(object):
         self.name = name
         self.api = api
 
-
     def create_collection(self, name):
         """
         """
@@ -205,7 +304,6 @@ class Collection(object):
 
         return collection
 
-
     def __init__(self, name, api, **kwargs):
         """
         """
@@ -216,7 +314,6 @@ class Collection(object):
         self.resource = api.collection
         self.api = api
 
-
     def setData(self, **kwargs):
         """
         """
@@ -251,7 +348,6 @@ class Collection(object):
         else:
             self.id = '0'
 
-
     def get(self):
         """
         """
@@ -262,14 +358,12 @@ class Collection(object):
 
         return data
 
-
     def create_document(self):
         """
         """
 
         return Document.create(collection=self)
 
-
     def create_edge(self, from_doc, to_doc, edge_data={}):
         """
         """
@@ -281,14 +375,16 @@ class Collection(object):
             edge_data=edge_data
         )
 
-
     def get_document_by_example(self, example_data):
         """
         """
 
         all_docs = []
 
-        result_dict = SimpleQuery.getByExample(collection=self.name, example_data=example_data)
+        result_dict = SimpleQuery.getByExample(
+            collection=self.name,
+            example_data=example_data
+        )
 
         if result_dict['count'] > 0:
             for result in result_dict['result']:
@@ -303,7 +399,6 @@ class Collection(object):
 
         return all_docs
 
-
     def documents(self):
         """
         """
@@ -315,7 +410,12 @@ class Collection(object):
             document_key = splitted_uri[-1]
             document_id = "%s/%s" % (self.name, document_key)
 
-            doc = Document(id=document_id, key=document_key, collection=self, api=self.api)
+            doc = Document(
+                id=document_id,
+                key=document_key,
+                collection=self,
+                api=self.api
+            )
             document_list.append(doc)
 
         return document_list
@@ -357,7 +457,6 @@ class Document(object):
 
         self.is_loaded = False
 
-
     def get(self):
         """
         """
@@ -367,14 +466,12 @@ class Document(object):
 
         return data
 
-
     def save(self):
         """
         """
 
         self.resource(self.id).patch(data=self.data)
 
-
     def getData(self, key):
         """
         """
@@ -386,15 +483,12 @@ class Document(object):
 
         return self.data[key]
 
-
     def setData(self, key, value):
         """
         """
 
         self.data[key] = value
 
-
-
     def __repr__(self):
         """
         """
@@ -443,4 +537,4 @@ class Edge(Document):
 
         super(Edge, self).__init__(id=id, key=key, collection=collection, api=api)
 
-        self.resource = api.edge
\ No newline at end of file
+        self.resource = api.edge
